package com.example.navigationbaato.activity

import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.location.Location
import android.os.Build
import android.os.Bundle
import android.os.VibrationEffect
import android.os.Vibrator
import android.util.Log
import android.view.View
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.transition.TransitionManager
import com.baato.baatolibrary.models.DirectionsAPIResponse
import com.baato.baatolibrary.services.BaatoRouting
import com.baato.baatolibrary.services.BaatoRouting.BaatoRoutingRequestListener
import com.example.navigationbaato.NavigationbaatoPlugin
import com.example.navigationbaato.R
import com.example.navigationbaato.utilities.PluginUtilities
import com.google.android.material.snackbar.BaseTransientBottomBar
import com.google.android.material.snackbar.Snackbar
import com.google.gson.Gson
import com.mapbox.android.core.location.LocationEngine
import com.mapbox.android.core.location.LocationEngineListener
import com.mapbox.android.core.location.LocationEnginePriority
import com.mapbox.android.core.location.LocationEngineProvider
import com.mapbox.api.directions.v5.models.DirectionsResponse
import com.mapbox.api.directions.v5.models.DirectionsRoute
import com.mapbox.geojson.Point
import com.mapbox.mapboxsdk.Mapbox
import com.mapbox.mapboxsdk.camera.CameraPosition
import com.mapbox.mapboxsdk.camera.CameraUpdateFactory
import com.mapbox.mapboxsdk.geometry.LatLng
import com.mapbox.mapboxsdk.geometry.LatLngBounds
import com.mapbox.mapboxsdk.maps.MapView
import com.mapbox.mapboxsdk.maps.MapboxMap
import com.mapbox.mapboxsdk.maps.OnMapReadyCallback
import com.mapbox.services.android.navigation.ui.v5.SoundButton
import com.mapbox.services.android.navigation.ui.v5.camera.DynamicCamera
import com.mapbox.services.android.navigation.ui.v5.instruction.InstructionView
import com.mapbox.services.android.navigation.ui.v5.map.NavigationMapboxMap
import com.mapbox.services.android.navigation.ui.v5.summary.SummaryBottomSheet
import com.mapbox.services.android.navigation.ui.v5.voice.NavigationSpeechPlayer
import com.mapbox.services.android.navigation.ui.v5.voice.SpeechAnnouncement
import com.mapbox.services.android.navigation.ui.v5.voice.SpeechPlayerProvider
import com.mapbox.services.android.navigation.v5.milestone.Milestone
import com.mapbox.services.android.navigation.v5.milestone.MilestoneEventListener
import com.mapbox.services.android.navigation.v5.milestone.VoiceInstructionMilestone
import com.mapbox.services.android.navigation.v5.navigation.MapboxNavigation
import com.mapbox.services.android.navigation.v5.navigation.MapboxNavigationOptions
import com.mapbox.services.android.navigation.v5.offroute.OffRouteListener
import com.mapbox.services.android.navigation.v5.routeprogress.ProgressChangeListener
import com.mapbox.services.android.navigation.v5.routeprogress.RouteProgress
import java.util.*


class NavigationActivity : AppCompatActivity(),
        OnMapReadyCallback,
        LocationEngineListener,
        ProgressChangeListener,
        MilestoneEventListener,
        OffRouteListener {

    var receiver: BroadcastReceiver? = null

//    private var navigationView: NavigationView? = null
//    private lateinit var navigationMapboxMap: NavigationMapboxMap
//    private lateinit var mapboxNavigation: MapboxNavigation
    private var dropoffDialogShown = false
    private var lastKnownLocation: Location? = null
    private var speechPlayer: NavigationSpeechPlayer? = null
    private var route: DirectionsRoute? = null
    private var points: MutableList<Point> = mutableListOf()

    private var currentDestination: Point? = null;
    private lateinit var locationEngine: LocationEngine;
    var navigationLayout: ConstraintLayout? = null
    var mapView: MapView? = null
    var instructionView: InstructionView? = null

    var summaryBottomSheet: SummaryBottomSheet? = null

    private var accessToken = ""

    private val FIRST = 0
    private val ONE_HUNDRED_MILLISECONDS = 100
    private val BOTTOMSHEET_PADDING_MULTIPLIER = 4
    private val TWO_SECONDS_IN_MILLISECONDS = 2000
    private val BEARING_TOLERANCE = 90.0
    private val LONG_PRESS_MAP_MESSAGE = "Long press the map to select a destination."
    private val SEARCHING_FOR_GPS_MESSAGE = "Searching for GPS..."
    private val ZERO_PADDING = 0
    private val DEFAULT_ZOOM = 12.0
    private val DEFAULT_TILT = 0.0
    private val DEFAULT_BEARING = 0.0
    private val ONE_SECOND_INTERVAL = 1000

    private var navigation: MapboxNavigation? = null

    var navigationMap: NavigationMapboxMap? = null
    private var lastLocation: Location? = null

    private var destination: Point? = null
    private var mapState: MapState? = null
    private var mapboxMap: MapboxMap? = null
    private var origin: Point? = null

    var directionsResponse: DirectionsResponse? = null
    private var soundButton: SoundButton? = null
    private var isMuted = false
    var initatingLoacation: Location? = null

    var mapStyle = "https://api.baato.io/api/v1/styles/breeze"
    var navMode = "bike"
    var locale = "en"

    private enum class MapState {
        INFO, NAVIGATION
    }

    override fun onCreate(savedInstanceState: Bundle?) {

        receiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context, intent: Intent) {
                finish()
                NavigationLauncher.cleanUpPreferences(applicationContext)
            }
        }
        registerReceiver(receiver, IntentFilter(NavigationLauncher.KEY_STOP_NAVIGATION))

        super.onCreate(savedInstanceState)

        // For styling the InstructionView
        setTheme(R.style.CustomInstructionView)
        setContentView(R.layout.component_navigation)
        accessToken = PluginUtilities.getResourceFromContext(this.applicationContext, "baato_access_token")
        Mapbox.getInstance(this.applicationContext, null)


        var p = intent.getSerializableExtra("waypoints") as? MutableList<Point>
        if(p != null)
        {
            points = p
            origin = Point.fromLngLat(points[0].longitude(), points[0].latitude())
            destination = Point.fromLngLat(points[1].longitude(), points[1].latitude())
        }


//        navigationView = findViewById(R.id.navigationView)
        mapView = findViewById(R.id.mapView)
        navigationLayout = findViewById(R.id.componentNavigationLayout)
        instructionView = findViewById(R.id.instructionView)
        summaryBottomSheet = findViewById(R.id.summaryBottomSheet)


        //    mapView.setStyleUrl("http://api-staging.baato.io/api/v1/styles/retro?key=" + Constants.token);

//        val extras = intent.extras
        mapStyle = NavigationbaatoPlugin.mapStyleUrl!!
        navMode = NavigationbaatoPlugin.navigationMode
        locale = NavigationbaatoPlugin.navigationLanguage
        mapView!!.setStyleUrl(mapStyle + "?key="  + accessToken)
        var initialRoute = intent.getSerializableExtra("route") as? String
        if(initialRoute != null) {
//            route = initialRoute
            val parseLocale = Locale(locale)
            val gson = Gson()
            val directionResponse: DirectionsAPIResponse = gson.fromJson(initialRoute, DirectionsAPIResponse::class.java)
            val pResponse = BaatoRouting.getParsedNavResponse(directionResponse , navMode, parseLocale, applicationContext)
            val directionsResponse = DirectionsResponse.fromJson(pResponse)
            route = directionsResponse.routes()[0]
        }
        mapView!!.onCreate(savedInstanceState)

        // Will call onMapReady
        mapView!!.getMapAsync(this)
        //    CustomNavigationNotification customNotification = new CustomNavigationNotification(getApplicationContext());
    }

    override fun onMapReady(mapboxMap: MapboxMap) {
//        if(route == null) {
//            getRoute(origin!!, destination!!, false)
//        }
        this.mapboxMap = mapboxMap
        //remove mapbox attribute
        mapboxMap.uiSettings.isAttributionEnabled = false
        mapboxMap.uiSettings.isLogoEnabled = false
        mapboxMap.setStyleUrl(mapStyle + "?key=" + accessToken
        ) { style: String? ->
            mapboxMap.cameraPosition = CameraPosition.Builder()
                    .target(LatLng(origin!!.latitude(), origin!!.longitude()))
                    .zoom(14.0)
                    .build()
            mapState = MapState.INFO
            navigationMap = NavigationMapboxMap(mapView!!, mapboxMap)
            // For voice instructions
            initializeSpeechPlayer()

            // For Location updates
            initializeLocationEngine()
            // For navigation logic / processing
            initializeNavigation(mapboxMap)
        }
    }

    fun onNavigationReady() {
        if(mapState == MapState.INFO && route == null) {
            getRoute(origin!!, destination!!, false)
        } else {
            // Transition to navigation state
            mapState = MapState.NAVIGATION

            // Show the InstructionView
            TransitionManager.beginDelayedTransition(navigationLayout!!)
            instructionView!!.visibility = View.VISIBLE
            summaryBottomSheet!!.visibility = View.VISIBLE
            //    navigation.addMilestoneEventListener(this);
            soundButton = instructionView!!.findViewById(R.id.soundLayout)
            // Start navigation
            adjustMapPaddingForNavigation()
            navigation!!.startNavigation(route!!)
            if (soundButton!!.hasOnClickListeners()) {
                soundButton!!.setOnClickListener { soundButton!!.toggleMute() }
            } else {
                soundButton!!.addOnClickListener { soundButton!!.toggleMute() }
            }
        }
    }

    /*
     * LocationEngine listeners
     */

    /*
     * LocationEngine listeners
     */
    @SuppressLint("MissingPermission")
    override fun onConnected() {
        locationEngine.requestLocationUpdates()
    }

    override fun onLocationChanged(location: Location) {
        if (lastLocation == null) {
            // Move the navigationMap camera to the first Location
            moveCameraTo(location)

            // Allow navigationMap clicks now that we have the current Location
            showSnackbar("Please wait...", BaseTransientBottomBar.LENGTH_LONG)
        }

        // Cache for fetching the route later
        updateLocation(location)
    }

    /*
     * Navigation listeners
     */

    /*
     * Navigation listeners
     */
    override fun onProgressChange(location: Location, routeProgress: RouteProgress?) {
        // Cache "snapped" Locations for re-route Directions API requests
        updateLocation(location)

        // Update InstructionView data from RouteProgress
        instructionView!!.update(routeProgress)
        summaryBottomSheet!!.update(routeProgress)
    }

    override fun onMilestoneEvent(routeProgress: RouteProgress?, instruction: String?, milestone: Milestone) {
        playAnnouncement(milestone)
    }

    override fun userOffRoute(location: Location?) {
        calculateRouteWith(destination!!, true)
        navigation!!.stopNavigation()
    }

    /*
     * Activity lifecycle methods
     */

    /*
     * Activity lifecycle methods
     */
    override fun onResume() {
        super.onResume()
        mapView!!.onResume()
    }

    override fun onPause() {
        super.onPause()
        mapView!!.onPause()
    }

    override fun onStart() {
        super.onStart()
        mapView!!.onStart()
        if (navigationMap != null) {
            navigationMap!!.onStart()
        }
    }



    override fun onStop() {
        super.onStop()
        mapView!!.onStop()
        if (navigationMap != null) {
            navigationMap!!.onStop()
        }
    }

    override fun onLowMemory() {
        super.onLowMemory()
        mapView!!.onLowMemory()
    }

    override fun onDestroy() {
        super.onDestroy()
        mapView!!.onDestroy()

        // Ensure proper shutdown of the SpeechPlayer
        if (speechPlayer != null) {
            speechPlayer!!.onDestroy()
        }

        // Prevent leaks
        removeLocationEngineListener()
        (navigation!!.cameraEngine as DynamicCamera).clearMap()
        // MapboxNavigation will shutdown the LocationEngine
        navigation!!.onDestroy()
    }

    private fun initializeSpeechPlayer() {
//        val parseLocale = Locale(locale)
//        val language = parseLocale.language
        val accessToken = "pk.xxx"
        val speechPlayerProvider = SpeechPlayerProvider(application, locale, true, accessToken)
        speechPlayer = NavigationSpeechPlayer(speechPlayerProvider)
    }

    private fun initializeLocationEngine() {
        val locationEngineProvider = LocationEngineProvider(this)
        locationEngine = locationEngineProvider.obtainLocationEngineBy(LocationEngine.Type.ANDROID)!!
        locationEngine.priority = LocationEnginePriority.HIGH_ACCURACY
        locationEngine.addLocationEngineListener(this)
        locationEngine.fastestInterval = ONE_SECOND_INTERVAL
        locationEngine.activate()
    }

    private fun initializeNavigation(mapboxMap: MapboxMap) {
        val options = MapboxNavigationOptions.builder()
            .defaultMilestonesEnabled(true)
            .build()
        navigation = MapboxNavigation(this, accessToken, options)
        navigation!!.locationEngine = locationEngine
        navigation!!.cameraEngine = DynamicCamera(mapboxMap)
        navigation!!.addProgressChangeListener(this)
        navigation!!.addMilestoneEventListener(this)
        navigation!!.addOffRouteListener(this)
        navigationMap!!.addProgressChangeListener(navigation)
        navigationMap!!.resetCameraPosition()
        onNavigationReady()
    }

    private fun showSnackbar(text: String, duration: Int) {
        Snackbar.make(navigationLayout!!, text, duration).show()
    }

    private fun playAnnouncement(milestone: Milestone) {
        if (milestone is VoiceInstructionMilestone) {
            val announcement = SpeechAnnouncement.builder()
                    .voiceInstructionMilestone(milestone)
                    .build()
            Log.d("Announcement", announcement.toString())
            speechPlayer!!.play(announcement)
        }
    }

    private fun updateLocation(location: Location) {
        lastLocation = location
        navigationMap!!.updateLocation(location)
    }

    private fun moveCameraTo(location: Location) {
        val cameraPosition = buildCameraPositionFrom(location, location.bearing.toDouble())
        navigationMap!!.retrieveMap().animateCamera(
                CameraUpdateFactory.newCameraPosition(cameraPosition), TWO_SECONDS_IN_MILLISECONDS
        )
    }

    private fun moveCameraToInclude(destination: Point) {
        val origin = LatLng(lastLocation)
        val bounds = LatLngBounds.Builder()
                .include(origin)
                .include(LatLng(destination.latitude(), destination.longitude()))
                .build()
        val resources = resources
        val routeCameraPadding = 56
        val padding = intArrayOf(routeCameraPadding, routeCameraPadding, routeCameraPadding, routeCameraPadding)
        val cameraPosition = navigationMap!!.retrieveMap().getCameraForLatLngBounds(bounds, padding)
        navigationMap!!.retrieveMap().animateCamera(
                CameraUpdateFactory.newCameraPosition(cameraPosition), TWO_SECONDS_IN_MILLISECONDS
        )
    }

    private fun moveCameraOverhead() {
        if (lastLocation == null) {
            return
        }
        val cameraPosition = buildCameraPositionFrom(lastLocation!!, DEFAULT_BEARING)
        navigationMap!!.retrieveMap().animateCamera(
                CameraUpdateFactory.newCameraPosition(cameraPosition), TWO_SECONDS_IN_MILLISECONDS
        )
    }

    private fun buildCameraPositionFrom(location: Location, bearing: Double): CameraPosition {
        return CameraPosition.Builder()
                .zoom(DEFAULT_ZOOM)
                .target(LatLng(location.latitude, location.longitude))
                .bearing(bearing)
                .tilt(DEFAULT_TILT)
                .build()
    }

    private fun adjustMapPaddingForNavigation() {
        val resources = resources
        val mapViewHeight = mapView!!.height
        val bottomSheetHeight = summaryBottomSheet!!.height
        val topPadding: Int = mapViewHeight - bottomSheetHeight * BOTTOMSHEET_PADDING_MULTIPLIER
        navigationMap!!.retrieveMap().setPadding(ZERO_PADDING, topPadding, ZERO_PADDING, ZERO_PADDING)
    }

    private fun resetMapAfterNavigation() {
        navigationMap!!.removeRoute()
        navigationMap!!.clearMarkers()
        navigation!!.stopNavigation()
        moveCameraOverhead()
    }

    private fun removeLocationEngineListener() {
        if (locationEngine != null) {
            locationEngine.removeLocationEngineListener(this)
        }
    }

    private fun addLocationEngineListener() {
        if (locationEngine != null) {
            locationEngine.addLocationEngineListener(this)
        }
    }

    private fun calculateRouteWith(destination: Point, isOffRoute: Boolean) {
        val origin = Point.fromLngLat(lastLocation!!.longitude, lastLocation!!.latitude)
        val bearing = java.lang.Float.valueOf(lastLocation!!.bearing).toDouble()
        getRoute(origin, destination, isOffRoute)
    }

    private fun getRoute(origin: Point, destination: Point, isOffRoute: Boolean) {
        val points = arrayOfNulls<String>(2)
        points[0] = origin.latitude().toString() + "," + origin.longitude()
        points[1] = destination.latitude().toString() + "," + destination.longitude()
        BaatoRouting(this)
                .setPoints(points)
                .setAccessToken(accessToken)
                .setMode(navMode) //eg bike, car, foot
                .setAlternatives(false) //optional parameter
                .setInstructions(true) //optional parameter
                .withListener(object : BaatoRoutingRequestListener {
                    override fun onSuccess(directionResponse: DirectionsAPIResponse) {
                        val navResponse = directionResponse.data[0]
                        val distanceInKm = navResponse.distanceInMeters / 1000
                        val time = navResponse.timeInMs / 1000
                        val parseLocale = Locale(locale)
                        val parsedNavigationResponse = BaatoRouting.getParsedNavResponse(directionResponse, navMode, parseLocale, applicationContext)
                        val directionsResponse = DirectionsResponse.fromJson(parsedNavigationResponse)
                        route = directionsResponse.routes()[0]
                        if (mapState == MapState.INFO) {
                            mapState = MapState.NAVIGATION
                            onNavigationReady()
                        }
                       else {
                            handleRoute(directionsResponse, isOffRoute)
                        }
                    }

                    override fun onFailed(t: Throwable) {
                        if (t.message != null && t.message!!.contains("Failed to connect")) Toast.makeText(applicationContext, "Please connect to internet to get the routes!", Toast.LENGTH_SHORT).show()
                    }
                })
                .doRequest()
    }

    private fun handleRoute(response: DirectionsResponse, isOffRoute: Boolean) {
        val routes = response.routes()
        if (!routes.isEmpty()) {
            route = routes[FIRST]
            navigationMap!!.drawRoute(route!!)
            if (isOffRoute) {
                navigation!!.startNavigation(route!!)
            }
        }
    }

    @SuppressLint("MissingPermission")
    private fun vibrate() {
        val vibrator = getSystemService(VIBRATOR_SERVICE) as Vibrator
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            vibrator.vibrate(VibrationEffect.createOneShot(ONE_HUNDRED_MILLISECONDS.toLong(), VibrationEffect.DEFAULT_AMPLITUDE))
        } else {
            vibrator.vibrate(ONE_HUNDRED_MILLISECONDS.toLong())
        }
    }
}
        